// $Id: kapp.h,v 1.14 1997/07/26 18:33:45 ettrich Exp $
//
// $Log: kapp.h,v $
// Revision 1.14  1997/07/26 18:33:45  ettrich
// Matthias: added a little "public:" before the color definitions. Otherwise
//    clients (for example kwm) cannot read the settings, so they are senseless
//    at present. Kwm needs this now, so I hope this is ok.
//
// Revision 1.13  1997/07/25 19:46:42  kalle
// SGI changes
//
// Revision 1.12  1997/07/18 05:49:16  ssk
// Taj: All kdecore doc now in javadoc format (hopefully).
//
// Revision 1.11  1997/06/29 18:26:36  kalle
// 29.06.97:	KConfig reads and writes string lists
// 			Torben's patches to ktoolbar.*, kurl.h
//
// 22.06.97:	KApplications save and restore position and size of their top
// (unstable)	level widget.
//
// Revision 1.10  1997/06/25 14:41:58  ssk
// Taj: Updated source documentation.
//
// Revision 1.9  1997/05/17 20:38:22  kalle
// Kalle:
// - Bugfix for KPanner (from Paul Kendall)
// - Better colors with kdisplay schemes (from Bernd Wuebben)
// - new behavior in KApplication::invokeHTMLHelp(): if the first
//   argument (the filename) is empty, the filename is defaulted to
// 	$KDEDIR/doc/HTML/<appname>/<appname>.html
// - KApplication::getCaption for Matthias added (breaks binary compatibility!)
//
// Revision 1.8  1997/05/15 20:33:20  wuebben
// Bernd: Added signals:
// kdisplayFontChanged()
// kdisplayPaletteChanged()
// kdisplayStyleChaned()
// Some applications such as kedit need to know when the Font, Palette,
// or Style ngeduy.
//
// Revision 1.7  1997/05/13 05:49:00  kalle
// Kalle: Default arguments for KConfig::read*Entry()
// app-specific config files don't start with a dot
// Bufgix for the bugfix in htmlobj.cpp (FontManager)
//
// Revision 1.6  1997/05/08 22:53:16  kalle
// Kalle:
// KPixmap gone for good
// Eliminated static objects from KApplication and HTML-Widget
//
// Revision 1.5  1997/05/02 16:46:34  kalle
// Kalle: You may now override how KApplication reacts to external changes
// KButton uses the widget default palette
// new kfontdialog version 0,5
// new kpanner by Paul Kendall
// new: KIconLoader
//
// Revision 1.4  1997/04/28 06:57:45  kalle
// Various widgets moved from apps to libs
// Added KSeparator
// Several bugs fixed
// Patches from Matthias Ettrich
// Made ksock.cpp more alpha-friendly
// Removed XPM-Loading from KPixmap
// Reaping zombie KDEHelp childs
// WidgetStyle of KApplication objects configurable via kdisplay
//
// Revision 1.3  1997/04/21 22:37:23  kalle
// Bug in Kconfig gefixed (schrieb sein app-spezifisches File nicht mehr)
// kcolordlg und kapp abgedated (von Martin Jones)
//
// Revision 1.2  1997/04/15 20:01:55  kalle
// Kalles changes for 0.8
//
// Revision 1.1.1.1  1997/04/13 14:42:41  cvsuser
// Source imported
//
// Revision 1.1.1.1  1997/04/09 00:28:07  cvsuser
// Sources imported
//
// Revision 1.10  1997/03/16 22:33:39  kalle
// appName()
//
// Revision 1.9  1997/03/09 17:28:32  kalle
// KTextStream -> QTextStream
//
// Revision 1.8  1997/01/15 21:46:26  kalle
// getConfig() and getConfigState() public (kfm needs them)
//
// Revision 1.7  1997/01/15 20:18:36  kalle
// merged changed from 0.52
//
// Revision 1.6  1996/12/14 13:52:46  kalle
// Some documentation to the constructors added.
//
// Revision 1.5  1996/12/14 13:49:58  kalle
// finding out the state the application config object is in
//
// Revision 1.4  1996/12/14 12:49:31  kalle
// method names start with a small letter
//
// Revision 1.3  1996/12/07 22:22:23  kalle
// Drag and Drop
//
// Revision 1.2  1996/12/01 11:20:23  kalle
// private members, GetConfig() inline
//
// Revision 1.1  1996/12/01 10:57:10  kalle
// Initial revision
//
//
// KApplication header file - base class for all KDE applications
//
// (C) 1996 Matthias Kalle Dalheimer <mda@stardivision.de>
//
// DND stuff by Torben Weis <weis@stud.uni-frankfurt.de>
// 09.12.96

#ifndef _KAPP_H
#define _KAPP_H

#ifndef _KCONFIG_H
#include <Kconfig.h>
#endif

#include <drag.h>

#include <qapp.h>
#include <qfile.h>
#include <qstrlist.h>

#define kapp KApplication::getKApplication()

/** 
* A base class for all KDE applications.
*
* KApplication provides the application with KDE defaults such as
* accelerators, common menu entries, a KConfig object
* etc. KApplication installs a signal handler for the SIGCHLD signal
* in order to avoid zombie children. If you want to catch this signal
* yourself or don't want it to be caught at all, you have set a new
* signal handler (or SIG_IGN) after KApplication's constructor has
* run. 
*
* @short A base class for all KDE applications.
* @author Matthias Kalle Dalheimer <kalle@kde.org>
* @version $Id: kapp.h,v 1.14 1997/07/26 18:33:45 ettrich Exp $
*/ 
class KApplication : public QApplication
{
  Q_OBJECT
public:
/** 
* Constructor. Pass command-line arguments. 
*
* A KConfig object is
* created that contains an application-specific config file whose
* name is "~/." + argv[0] + "rc". This constructor should be considered
* obsolete. The state of the application-specific config file may be
* queried afterwards with getConfigState(). 
*/
  KApplication( int& argc, char** argv );

/** 
* Constructor. Pass command-line arguments. 
*
* A KConfig object is
* created that contains an application-specific config file whose
* name is "~/." + rAppName + "rc". The state of the application-specific 
* config file may be queried afterwards with getConfigState(). 
*/
  KApplication( int& argc, char** argv, const QString& rAppName );

/** 
* Destructor 
*/
  virtual ~KApplication();

/** Restore previous main window geometry. 
*
* If there is no previous size (e.g. first app start or deleted config 
* file), nothing is done. 
*/ 
  void restoreTopLevelGeometry() const;

/** 
* Return the current application object.
*
* This is similar to the global QApplication pointer qApp. It allows access
* to the single global KApplication object, since more than one cannot be
* created in the same application. It saves the trouble of having to pass
* the pointer to it explicitly to every function that may require it.
*/
  static KApplication* getKApplication() { return KApp; }

/** 
* Return the logical application name as set in the constructor.
*/
  const QString& appName() const { return aAppName; }
  
/** 
* Retrieve the application config object. 
*
* @return a pointer to the application's global KConfig object.
* @see KConfig
*/
  KConfig* getConfig() const { return pConfig; }

/** 
* Possible return values for getConfigState().
*
* @see #getConfigState
*/
  enum ConfigState { APPCONFIG_NONE, APPCONFIG_READONLY, 
					 APPCONFIG_READWRITE };

/** 
* Retrieve the state of the app-config object. 
*
* Possible return values
* are APPCONFIG_NONE (the application-specific config file could not be
* opened neither read-write nor read-only), APPCONFIG_READONLY (the
* application-specific config file is opened read-only, but not
* read-write) and APPCONFIG_READWRITE (the application-specific config
* file is opened read-write).
*
* @see #ConfigState
*/
  ConfigState getConfigState() const { return eConfigState; }

/**
* Invoke the kdehelp HTML help viewer. 
*
* @param aFilename	The filename that is to be loaded. Its location
*			is computed automatically according to the KFSSTND. 
*			If aFilename is empty, the logical appname with .html 
*			appended to it is used.
* @param aTopic		This allows context-sensitive help. Its value
*			will be appended to the filename, prefixed with
*			a "#" (hash) character.
*/
  void invokeHTMLHelp( QString aFilename, QString aTopic ) const;

/** 
* Get the KDE base dir. 
*
* This is the value of the KDEDIR
* environment variable if it is set in the process' environment,
* the compile time default of, if this was not present, either,
* /usr/local/kde. 
*/
  static QString kdedir();

/** 
* Find a file using standard KDE search paths. 
*
* Possible search paths
* include $KDEDIR, $KDEPATH, and "[KDE Setup]:Path=" entry in a config
* file. If file is not found, isEmpty() will return True
*/
  static QString findFile( const char *file );

/** 
* Get the KDE font list.
*
* This method allows you to get the KDE font 
* list which was composed by the user with kfontmanager. Usually you should 
* work only with those fonts in your kapplication. 
*  
*  @return true on success.
*/
  bool getKDEFonts(QStrList *fontlist);


/**
* Return a text for the window caption.
*
* This would be set either by
* "-caption", otherwise it will be equivalent to the name of the
* executable.
*/
   const char* getCaption() const;

  /**
  * An X11 atom used for IPC
  */
  Atom getDndSelectionAtom() { return DndSelection; }
  /**
  * An X11 atom used for IPC
  */
  Atom getDndProtocolAtom() { return DndProtocol; }    
  /**
  * An X11 atom used for IPC
  */
  Atom getDndEnterProtocolAtom() { return DndEnterProtocol; }    
  /**
  * An X11 atom used for IPC
  */
  Atom getDndLeaveProtocolAtom() { return DndLeaveProtocol; }    
  /**
  * An X11 atom used for IPC
  */
  Atom getDndRootProtocolAtom() { return DndRootProtocol; }    

  /**
  * Get the X11 display
  */
  Display *getDisplay() { return display; }
  /**
  * Used by KDNDDropZone to register
  */
  virtual void addDropZone( KDNDDropZone *_z ) { dropZones.append( _z ); }
  /**
  * Used by KDNDDropZone during shutdown
  */
  virtual void removeDropZone( KDNDDropZone *_z ) { dropZones.remove( _z ); }
  /**
  * Set the DropZone which reveives root drop events.
  */
  virtual void setRootDropZone( KDNDDropZone *_z ) { rootDropZone = _z; }

protected:
/**
* List of all DropZones.
*/
  QList<KDNDDropZone> dropZones;

/** 
* The last drop zone the mouse was over.
*
* If we get a DndLeaveProtocol we must inform 'lastEnteredDropZone'
* that the mouse left the DropZone.
*/
  KDNDDropZone *lastEnteredDropZone;

/**
* The DropZone which receives root drop events.
*/
  KDNDDropZone *rootDropZone;

/**
* Used to catch X11 events
*/
  bool x11EventFilter( XEvent * );

/**
* An X11 atom used for IPC
*/
  Atom DndSelection;
/**
* An X11 atom used for IPC
*/
  Atom DndProtocol;
/**
* An X11 atom used for IPC
*/
  Atom DndEnterProtocol;
/**
* An X11 atom used for IPC
*/
  Atom DndLeaveProtocol;
/**
* An X11 atom used for IPC
*/
  Atom DndRootProtocol;
/**
* The X11 display
*/
  Display *display;

  Atom KDEChangePalette;
  Atom KDEChangeGeneral;
  Atom KDEChangeStyle;

  /// Current application object.
  static KApplication *KApp;

private:
  KConfig* pConfig; // application config object
  QTextStream* pConfigStream; // stream of the application-specific config file
  QFile* pConfigFile; // application-specific config file
  QString aAppName; // logical application name
  QString aCaption; // the name for the window title
  ConfigState eConfigState;
  static QStrList* pSearchPaths;

  void init( );
  void parseCommandLine( int&, char** ); // search for special KDE arguments

  void buildSearchPaths();
  void appendSearchPath( const char *path );

  virtual void kdisplaySetPalette();
  virtual void kdisplaySetStyle();
  virtual void kdisplaySetFont();
  virtual void kdisplaySetStyleAndFont();
  virtual void readSettings();
  void resizeAll();
  virtual void KApplication::applyGUIStyle(GUIStyle newstyle);

public:

  QColor inactiveTitleColor;
  QColor inactiveTextColor;
  QColor activeTitleColor;
  QColor activeTextColor;
  QColor backgroundColor;
  QColor textColor;
  QColor selectColor;
  QColor selectTextColor;
  QColor windowColor;
  QColor windowTextColor;
  QFont generalFont;
  GUIStyle applicationStyle;

signals:
/** 
* KApplication has changed its Palette due to a KDisplay request.
*
* Normally, widgets will update their palettes automatically, but you
* should connect to this to program special behaviour.
*/
  void kdisplayPaletteChanged();

/** 
* KApplication has changed its GUI Style due to a KDisplay request.
*
* Normally, widgets will update their styles automatically (as they would
* respond to an explicit setGUIStyle() call), but you should connect to
* this to program special behaviour.
*/
  void kdisplayStyleChanged();

/**
* KApplication has changed its Font due to a KDisplay request.
*
* Normally widgets will update their fonts automatically, but you should
* connect to this to monitor global font changes, especially if you are
* using explicit fonts.
*/
  void kdisplayFontChanged();
};

#endif
