#include <qpainter.h>
#include <qdrawutl.h>
#include <qpalette.h>
#include <qstring.h>
#include <qframe.h>

#include "ktopwidget.h"

#include "kmenubar.moc"

#include <klocale.h>
#define klocale KLocale::klocale()

#define CONTEXT_TOP 1
#define CONTEXT_BOTTOM 2
#define CONTEXT_FLOAT 3

// uncomment this to have menubar raised:
//  #define MENUBAR_IS_RAISED

// But beware that it looks ugly when toolmar is raised
// I cannot draw shaded panel around menubar, cause it would
// totaly #%$*% resizing
// If kde comunity wants to have raised menubar do the
// resizing things properly or just uncoment this and suffer uglyness

_menuBar::_menuBar (QWidget *parent, const char *name)
  : QMenuBar (parent, name)
 {
#ifndef MENUBAR_IS_RAISED
   setFrameStyle(NoFrame);
#endif
 }

_menuBar::~_menuBar ()
 {
 }

/*************************************************************/

KMenuBar::KMenuBar(QWidget *parent, const char *name)
  : QFrame( parent, name )
{
  Parent = parent;        // our father
  title = 0;

  frame = new QFrame (this);
  frame->setFrameStyle(NoFrame);
  menu = new _menuBar (frame);
  connect (menu, SIGNAL(activated(int)), this, SLOT(slotActivated(int)));
  connect (menu, SIGNAL(highlighted(int)), this, SLOT(slotHighlighted(int)));
  init();
}


void KMenuBar::resizeEvent (QResizeEvent *)
{
  if (position == Floating) // What now?
   {
     // Khm... I'm resized from kwm
     // menu bar installs eventFilter on parent, so we don't have
     // to bother with resizing her
     frame->setGeometry(11, 0, width()-11, height());
     frame->resize(menu->width(), menu->height());
     if (height() != frame->height() ||
         width() != frame->width()+11)
      {
        //warning ("resize");
        resize(frame->width()+11, frame->height());
      }
   }
  else
   {
     // I will be resized from KtopLevel
     frame->setGeometry (11, 0, width()-11, height());
     if (menu->height() != height())
      {
        frame->resize(frame->width(), menu->height());
        resize(width(), menu->height());
      }
   }
}

void KMenuBar::ContextCallback( int index )
{
  switch ( index )
   {
    case CONTEXT_TOP:
      setMenuBarPos( Top );
      break;
    case CONTEXT_BOTTOM:
      setMenuBarPos( Bottom );
      break;
    case CONTEXT_FLOAT:
      if (position == Floating)
        setMenuBarPos (lastPosition);
      else
        setMenuBarPos( Floating );
      break;
   }
}

void KMenuBar::init()
{
  context = new QPopupMenu( 0, "context" );
  context->insertItem( klocale->translate("Top"),  CONTEXT_TOP );
  context->insertItem( klocale->translate("Bottom"), CONTEXT_BOTTOM );
  context->insertItem( klocale->translate("Floating"), CONTEXT_FLOAT );
  connect( context, SIGNAL( activated( int ) ), this,
	   SLOT( ContextCallback( int ) ) );
  
  position = Top;
  moving = TRUE;
  //setFrameStyle( QFrame::Panel | QFrame::Raised );
  setLineWidth( 0 );
  resize( Parent->width(), menu->height());
  enableFloating (TRUE);
}


KMenuBar::~KMenuBar()
{
  if (position == Floating)
     recreate (Parent, oldWFlags, QPoint (oldX, oldY), TRUE);
  delete context;
  debug ("KMenuBar Destructor: finished");
}

void KMenuBar::mousePressEvent ( QMouseEvent *m )
{
  if ( moving )
    context->popup( mapToGlobal( m->pos() ), 0 );
}

void KMenuBar::paintEvent(QPaintEvent *)
{
  int offset=3;
  QColorGroup g = QWidget::colorGroup();
  QPainter *paint = new QPainter();

  int menubarHeight = menu->height();
  
  paint->begin( this );

  // Handle point

  switch ( position )
   {
    case Top:
    case Bottom:
      qDrawShadePanel( paint, offset, 3, 4, menubarHeight-8,
                       g , FALSE, 2);
      offset+=4;
      qDrawShadePanel( paint, offset, 3, 4, menubarHeight-8,
		       g , FALSE, 2);
      offset+=9;
      break;

    case Floating:
      qDrawShadePanel( paint, offset, 3, 4, menubarHeight-8,
                       g , FALSE, 2);
      offset+=4;
      qDrawShadePanel( paint, offset, 3, 4, menubarHeight-8,
                       g , FALSE, 2);
      offset+=9;
      break;
   }

#ifdef MENUBAR_IS_RAISED
  qDrawShadePanel(paint, 0, 0, width(), height(), g , FALSE, 2);
#endif
  paint->end();
  delete paint;
}

void KMenuBar::closeEvent (QCloseEvent *e)
{
  if (position == Floating)
   {
     position = lastPosition;
     recreate (Parent, oldWFlags, QPoint (oldX, oldY), TRUE);
     context->changeItem (klocale->translate("Float"), CONTEXT_FLOAT);
     emit moved (position);
     e->ignore();
     return;
   }
  e->accept();
}

void KMenuBar::enableMoving(bool flag)
{
  moving = flag; 
}

void KMenuBar::setMenuBarPos(menuPosition pos)
{
  if (position != pos)
   {
     if (pos == Floating)
      {
        lastPosition = position;
        position = pos;
        oldX = x();
        oldY = y();
        oldWFlags = getWFlags();
        hide();
        recreate(0, WStyle_Customize | WStyle_Title | WStyle_Minimize | WStyle_DialogBorder,
                 QCursor::pos(), TRUE);
        //updateRects (TRUE);
        //show();
        if (title != 0)
          setCaption (title);
        context->changeItem (klocale->translate("UnFloat"), CONTEXT_FLOAT);
        emit moved (pos);
        return;
      }
     else if (position == Floating) // was floating
      {
        position = pos;
        hide();
        recreate(Parent, oldWFlags, QPoint(oldX, oldY), TRUE);
        //updateRects (TRUE);
        context->changeItem (klocale->translate("Float"), CONTEXT_FLOAT);
        emit moved (pos);
        return;
      }
     else
      {
        position = pos;
        emit moved ( pos );
        return;
      }
   }
}

void KMenuBar::enableFloating (bool arrrrrrgh)
{
  context->setItemEnabled (CONTEXT_FLOAT, arrrrrrgh);
}

/*******************************************************/

uint KMenuBar::count()
{
  return menu->count();
}

int KMenuBar::insertItem(const char *text,
               const QObject *receiver, const char *member,
               int accel)
{
  return menu->insertItem(text, receiver, member, accel);
}

int KMenuBar::insertItem( const char *text, int id, int index)
{
  return menu->insertItem(text, id, index);
}
int KMenuBar::insertItem( const char *text, QPopupMenu *popup,
                          int id, int index)
{
  return menu->insertItem(text, popup, id, index);
}

void KMenuBar::insertSeparator(int index)
{
  menu->insertSeparator(index);
}

void KMenuBar::removeItem( int id )
{
  menu->removeItem(id);
}
void KMenuBar::removeItemAt( int index )
{
  menu->removeItemAt(index);
}
void KMenuBar::clear()
{
  menu->clear();
}

int KMenuBar::accel( int id )
{
  return menu->accel(id);
}
void KMenuBar::setAccel( int key, int id )
{
  menu->setAccel(key, id);
}

const char *KMenuBar::text( int id )
{
  return menu->text(id);
}

void KMenuBar::changeItem( const char *text, int id )
{
  menu->changeItem(text, id);
}

void KMenuBar::setItemChecked(int id , bool flag)
{
  menu->setItemChecked(id , flag);
}

void KMenuBar::setItemEnabled(int id, bool flag)
{
  menu->setItemEnabled(id, flag);
}


void KMenuBar::slotActivated (int id)
{
  emit activated(id);
}

void KMenuBar::slotHighlighted (int id)
{
  emit highlighted (id);
}
